package com.css.simulation.resource.scheduler.service;

import api.common.pojo.constants.DictConstants;
import api.common.pojo.param.RedisParameter;
import api.common.util.CollectionUtil;
import api.common.util.StringUtil;
import api.common.util.TimeUtil;
import com.css.simulation.resource.scheduler.feign.CommonService;
import com.css.simulation.resource.scheduler.manager.TaskIndexManager;
import com.css.simulation.resource.scheduler.manager.TaskManager;
import com.css.simulation.resource.scheduler.mapper.IndexTemplateMapper;
import com.css.simulation.resource.scheduler.mapper.ProjectMapper;
import com.css.simulation.resource.scheduler.mapper.TaskIndexMapper;
import com.css.simulation.resource.scheduler.mapper.TaskMapper;
import com.css.simulation.resource.scheduler.pojo.po.IndexTemplatePO;
import com.css.simulation.resource.scheduler.pojo.po.ProjectPO;
import com.css.simulation.resource.scheduler.pojo.po.TaskIndexPO;
import com.css.simulation.resource.scheduler.pojo.po.TaskPO;
import com.css.simulation.resource.scheduler.pojo.to.ScoreTO;
import com.css.simulation.resource.scheduler.util.ScoreUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class TaskService {

    private final String USER_ID = "simulation-resource-scheduler";

    @Autowired
    private CommonService commonService;
    @Autowired
    private ProjectMapper projectMapper;
    @Autowired
    private TaskManager taskManager;
    @Autowired
    private TaskMapper taskMapper;
    @Autowired
    private TaskIndexManager taskIndexManager;
    @Autowired
    private TaskIndexMapper taskIndexMapper;
    @Autowired
    private IndexTemplateMapper indexTemplateMapper;
    @Value("${spring.kafka.consumer.topic.manual-project}")
    private String manualProjectTopic;

    public void taskTick(String taskId) {
        // 刷新 redis 心跳时间
        ProjectPO projectPO = projectMapper.selectById(taskId);
        String projectId = projectPO.getId();
        commonService.set(RedisParameter.builder()
                .key(manualProjectTopic + ":" + projectId + ":" + taskId)
                .value(TimeUtil.getNow() + "")
                .build());
    }

    @Transactional
    public void taskState(String taskId, String state) {
        //1 根据 taskId 修改任务状态 taskState。
        taskMapper.updateState(taskId, state);
        //2 如果 taskState 为完成状态，校验一下项目的已完成数量。
        ProjectPO projectPO = projectMapper.selectById(taskId);
        String projectId = projectPO.getId();
        String scenePackageId = projectPO.getScenePackageId();  // 场景测试包 id，指标根 id
        int taskNum = projectMapper.selectTaskNumById(projectId);
        int completedTaskNum = projectMapper.selectTaskNumByProjectIdAndState(projectId, DictConstants.TASK_COMPLETED);
        if (taskNum != completedTaskNum) {
            return;
        }  //3 如果已完成任务数等于所有任务数量，调用打分程序，对项目中每个指标进行打分，
        List<TaskPO> taskList = taskMapper.selectTaskListByProjectId(projectId);  // 所有任务信息
        List<IndexTemplatePO> leafIndexTemplateList = indexTemplateMapper.selectLeafIndexWithRuleDetailsByPackageId(scenePackageId);
        List<TaskIndexPO> leafTaskIndexList = new ArrayList<>();

        // 计算所有叶子节点指标的得分
        for (IndexTemplatePO indexTemplatePO : leafIndexTemplateList) {
            String ruleDetails = indexTemplatePO.getRuleDetails();
            Set<String> sceneIdSet = new HashSet<>();
            String naturalIds = indexTemplatePO.getSceneNaturalIds();
            String standardIds = indexTemplatePO.getSceneStatueIds();
            String accidentIds = indexTemplatePO.getSceneTrafficIds();
            if (StringUtil.isNotEmpty(naturalIds)) {
                String[] naturalIdArray = naturalIds.split(",");
                sceneIdSet.addAll(Arrays.asList(naturalIdArray));
            }
            if (StringUtil.isNotEmpty(standardIds)) {
                String[] standardArray = standardIds.split(",");
                sceneIdSet.addAll(Arrays.asList(standardArray));
            }
            if (StringUtil.isNotEmpty(accidentIds)) {
                String[] accidentIdArray = accidentIds.split(",");
                sceneIdSet.addAll(Arrays.asList(accidentIdArray));
            }
            int resultNumberOfCurrentIndex = sceneIdSet.size();
            double sum = taskList.stream()
                    .filter(task1 -> sceneIdSet.contains(task1.getSceneId()))
                    .mapToDouble(task2 -> {
                        // 计算每个任务的得分
                        ScoreTO score = ScoreUtil.score(task2.getRunResult(), ruleDetails);
                        task2.setReturnSceneId(score.getUnitSceneID());
                        task2.setScore(score.getUnitSceneScore());
                        task2.setTargetEvaluate(score.getEvaluateItem());
                        task2.setScoreExplain(score.getScoreDescription());
                        task2.setModifyUserId(USER_ID);
                        task2.setModifyTime(TimeUtil.getNowForMysql());
                        return score.getUnitSceneScore();
                    }).sum();
            long notStandardSceneNum = taskList.stream()    // 计算不合格的任务数（不到100分就是不合格）
                    .filter(task1 -> sceneIdSet.contains(task1.getSceneId()) && task1.getScore() < 100)
                    .count();
            double leafIndexScore = sum / resultNumberOfCurrentIndex;   // 叶子指标的得分
            indexTemplatePO.setTempScore(leafIndexScore);
            TaskIndexPO leafTaskIndex = TaskIndexPO.builder()
                    .id(StringUtil.getRandomUUID())
                    .pId(projectId)
                    .target(indexTemplatePO.getIndexId())
                    .notStandardSceneNum((int) notStandardSceneNum)
                    .score(leafIndexScore)
                    .build();
            leafTaskIndex.setCreateUserId(USER_ID);
            leafTaskIndex.setCreateTime(TimeUtil.getNowForMysql());
            leafTaskIndex.setModifyUserId(USER_ID);
            leafTaskIndex.setModifyTime(TimeUtil.getNowForMysql());
            leafTaskIndex.setIsDeleted("0");
            leafTaskIndexList.add(leafTaskIndex);
        }

        //4 根据每个指标的得分和权重算出 project 的总得分。
        double totalScore = compute(leafIndexTemplateList);
        //5 保存分数
        //5-1 保存任务分数
        taskManager.batchUpdateByScoreResult(taskList);
        //5-2 保存指标分数
        taskIndexManager.batchInsertLeafIndex(leafTaskIndexList);
        //5-3 保存总分数
        TaskIndexPO totalTaskIndex = TaskIndexPO.builder()
                .id(StringUtil.getRandomUUID())
                .pId(projectId)
                .target(scenePackageId)
                .score(totalScore)
                .build();
        totalTaskIndex.setCreateUserId(USER_ID);
        totalTaskIndex.setCreateTime(TimeUtil.getNowForMysql());
        totalTaskIndex.setModifyUserId(USER_ID);
        totalTaskIndex.setModifyTime(TimeUtil.getNowForMysql());
        totalTaskIndex.setIsDeleted("0");
        taskIndexMapper.insertTotalIndex(totalTaskIndex);
    }


    public Boolean taskConfirm(String taskId) {
        // 查询 task 如果不是 pending 则不执行
        String state = taskMapper.selectStateById(taskId);
        return DictConstants.TASK_PENDING.equals(state);
    }

    public double compute(List<IndexTemplatePO> leaf) {
        double result = 0.0;
        Map<String, List<IndexTemplatePO>> groups = leaf.stream().collect(Collectors.groupingBy(IndexTemplatePO::getParentId));
        Set<String> idSet = groups.keySet();
        List<IndexTemplatePO> indexTemplatePOList = indexTemplateMapper.selectByIdList(CollectionUtil.setToList(idSet));
        indexTemplatePOList.forEach(index1 -> {
            double sum = groups.get(index1.getIndexId()).stream().mapToDouble(index2 -> index2.getTempScore() * Double.parseDouble(index2.getWeight())).sum();
            index1.setTempScore(sum);
        });
        if (indexTemplatePOList.size() > 1) {
            result = result + compute(indexTemplatePOList);
        }
        return result;
    }
}
